/*
 * Copyright (C) 2010 Neil Jagdish Patel
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <njpatel@gmail.com>
 */

namespace GwibberGtk
{
  public class StreamViewTile : Gtk.HBox
  {
    private Gtk.Alignment alignment;
    private Gtk.Image  icon;
    private Gtk.VBox   vbox;
    private new Gtk.Label  name;
    private Gtk.Label  time;
    private Gtk.Label  message;

    public uint uid = 0;

    public StreamViewTile ()
    {
      Object (homogeneous:false, spacing:12);
      set_border_width (4);
    }

    construct
    {
      alignment = new Gtk.Alignment (0.5f, 0.0f, 0.0f, 0.0f);
      pack_start (alignment, false, false, 0);

      icon = new Gtk.Image.from_icon_name ("stock_person", Gtk.IconSize.DIALOG);
      alignment.add (icon);

      vbox = new Gtk.VBox (false, 0);
      pack_start (vbox, true, true, 0);
      
      var hbox = new Gtk.HBox (false, 0);
      vbox.pack_start (hbox, false, false, 2);
      
      name = new Gtk.Label ("");
      name.set_markup ("<b></b>");
      name.set_alignment (0.0f, 0.5f);
      hbox.pack_start (name, true, true, 0);

      time = new Gtk.Label ("");
      time.set_markup ("<span font_weight='light' font_size='small'>24 mins</span>");
      time.set_alignment (0.0f, 1.0f);
      hbox.pack_start (time, false, false, 0);

      message = new Gtk.Label ("");
      message.set_single_line_mode (false);
      message.set_line_wrap (true);
      message.set_line_wrap_mode (Pango.WrapMode.WORD_CHAR);
      message.set_alignment (0.0f, 0.0f);
      vbox.pack_start (message, false, false, 2);
    }

    public void reset ()
    {
      hide ();
    }

    public void set_details (string _account,
                             string _stream,
                             string _service,
                             string _sender,
                             string _timestamp,
                             string _message,
                             string _icon_uri,
                             string _url,
                             string _html,
                             string _timestring)
    {
      Gtk.Allocation alloc;

      name.set_markup ("<b>" + _sender + "</b>");

      icon.set_from_file(_icon_uri);

      try 
      {
        message.set_markup (_html);
      }
      finally
      {
        debug ("MESSAGE: fallback to _message");
        message.set_markup (_message);
      }

      time.set_markup (_timestring);

      vbox.get_allocation (out alloc);
      message.set_size_request (alloc.width, -1);

      queue_resize ();
      show ();
    }                        
  }
}
